@@ -130,6 +130,23 @@ static NSMutableDictionary *_modelMap = nil;
   return [_modelMap objectForKey: type];
 }
 
++ (NSArray *) supportedTypes
+{
+  NSArray *objectArray = [_modelMap allValues];
+  NSArray *sortedArray = [objectArray sortedArrayUsingSelector: 
+					    @selector(_comparePriority:)];
+  NSEnumerator *oen = [sortedArray objectEnumerator];
+  Class cls = nil;
+  NSMutableArray *types = [[NSMutableArray alloc] init];
+
+  while ((cls = [oen nextObject]) != nil)
+    {
+      [types addObject: [cls type]];
+    }
+
+  return AUTORELEASE(types);
+}
+
 + (NSString *) supportedModelFileAtPath: (NSString *)modelPath
 {
   NSString *result = nil;
@@ -168,7 +185,7 @@ static NSMutableDictionary *_modelMap = nil;
   return result;
 }
 
-+ (GSModelLoader *)modelLoaderForFileType: (NSString *)type
++ (GSModelLoader *) modelLoaderForFileType: (NSString *)type
 {
   Class aClass = [GSModelLoaderFactory classForType: type];
   GSModelLoader *loader = nil;
@@ -186,7 +203,7 @@ static NSMutableDictionary *_modelMap = nil;
   return loader;
 }
 
-+ (GSModelLoader *)modelLoaderForFileName: (NSString *)modelPath
++ (GSModelLoader *) modelLoaderForFileName: (NSString *)modelPath
 {
   NSString *path = [GSModelLoaderFactory supportedModelFileAtPath: modelPath];
   GSModelLoader *result = nil;
