@@ -681,7 +681,7 @@ _attributesAtIndexEffectiveRange(
 
   if (range.length == 0)
     {
-      NSWarnMLog(@"Attempt to set attribute for zero-length range", 0);
+      NSWarnMLog(@"Attempt to set attribute for zero-length range");
       return;
     }
   if (attributes == nil)
