@@ -32,12 +32,12 @@
 #import <Foundation/NSException.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSKeyedArchiver.h>
+#import <Foundation/NSKeyValueCoding.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSValue.h>
 #import <Foundation/NSXMLParser.h>
 #import <Foundation/NSXMLDocument.h>
 #import <Foundation/NSXMLElement.h>
-#import	<GNUstepBase/GSMime.h>
 
 #import "AppKit/NSApplication.h"
 #import "AppKit/NSNib.h"
@@ -329,20 +329,18 @@
             } 	 
           else 	 
             { 	 
-              const char *nam = [label cString]; 	 
-              const char *type; 	 
-              unsigned int size; 	 
-              int offset; 	 
-              
-              /* 	 
-               * Use the GNUstep additional function to set the instance 	 
-               * variable directly. 	 
-               * FIXME - need some way to do this for libFoundation and 	 
-               * Foundation based systems. 	 
+              /*
+               * We cannot use the KVC mechanism here, as this would always retain _dst
+               * and it could also affect _setXXX methods and _XXX ivars that aren't
+               * affected by the Cocoa code.
                */ 	 
-              if (GSObjCFindVariable(source, nam, &type, &size, &offset)) 	 
-                { 	 
-                  GSObjCSetVariable(source, offset, size, (void*)&destination);
+              const char *name = [label cString];
+              Class class = object_getClass(source);
+              Ivar ivar = class_getInstanceVariable(class, name);
+              
+              if (ivar != 0)
+                {
+                  object_setIvar(source, ivar, destination);
                 }
             } 	 
 	}
@@ -410,7 +408,7 @@
                               __PRETTY_FUNCTION__,
                               NSStringFromClass([self class])];
           [NSException raise: NSInvalidArgumentException
-                      format: format];
+                      format: @"%@", format];
         }
       
       // Load the connection ID....
@@ -433,7 +431,7 @@
               NSString *format = [NSString stringWithFormat:@"%s:class: %@ - connection ID is missing or zero!",
                                   __PRETTY_FUNCTION__, NSStringFromClass([self class])];
               [NSException raise: NSInvalidArgumentException
-                          format: format];
+                          format: @"%@", format];
             }
         }
       else
@@ -442,7 +440,7 @@
                               __PRETTY_FUNCTION__,
                               NSStringFromClass([self class])];
           [NSException raise: NSInvalidArgumentException
-                      format: format];
+                      format: @"%@", format];
         }
     }
   else
@@ -582,7 +580,7 @@
                               __PRETTY_FUNCTION__,
                               NSStringFromClass([self class])];
           [NSException raise: NSInvalidArgumentException
-                      format: format];
+                      format: @"%@", format];
         }
       
       if ([coder containsValueForKey: @"object"])
@@ -632,7 +630,7 @@
 
 - (NSString *) description
 {
-  return [NSString stringWithFormat: @"<%@, %@, %@, %d>",
+  return [NSString stringWithFormat: @"<%@, %@, %@, %p>",
 		   [self className],
 		   object,
 		   parent,
@@ -857,11 +855,29 @@
       value = [properties objectForKey: @"IBAttributePlaceholdersKey"];
       if (value != nil)
         {
-          IBToolTipAttribute *tta = [(NSDictionary*)value objectForKey: @"ToolTip"];
+          NSDictionary *infodict = (NSDictionary*)value;
+          
+          // Process tooltips...
+          IBToolTipAttribute *tooltip = [infodict objectForKey: @"ToolTip"];
 
-          if ([realObj respondsToSelector: @selector(setToolTip:)])
+          if (tooltip && [realObj respondsToSelector: @selector(setToolTip:)])
             {
-              [realObj setToolTip: [tta toolTip]];
+              [realObj setToolTip: [tooltip toolTip]];
+            }
+          
+          // Process XIB runtime attributes...
+          if ([infodict objectForKey:@"IBUserDefinedRuntimeAttributesPlaceholderName"])
+            {
+              IBUserDefinedRuntimeAttributesPlaceholder *placeholder =
+                [infodict objectForKey:@"IBUserDefinedRuntimeAttributesPlaceholderName"];
+              NSArray *attributes = [placeholder runtimeAttributes];
+              NSEnumerator *objectIter = [attributes objectEnumerator];
+              IBUserDefinedRuntimeAttribute *attribute;
+              
+              while ((attribute = [objectIter nextObject]) != nil)
+                {
+                  [realObj setValue: [attribute value] forKeyPath: [attribute keyPath]];
+                }
             }
         }
 
@@ -1018,7 +1034,7 @@
     }
   NS_HANDLER
     {
-      NSLog(@"Exception occured while loading model: %@",[localException reason]);
+      NSLog(@"Exception occurred while loading model: %@",[localException reason]);
       // TEST_RELEASE(unarchiver);
     }
   NS_ENDHANDLER
@@ -1061,10 +1077,13 @@
 
 - (NSData *) _preProcessXib: (NSData *)data
 {
-  NSData *result = data;
+  NSData *result = nil;
+
+#if     GNUSTEP_BASE_HAVE_LIBXML
   NSXMLDocument *document = [[NSXMLDocument alloc] initWithData:data
 							options:0
 							  error:NULL];
+  result = data;
   if (document == nil)
     {
       NSLog(@"%s:DOCUMENT IS NIL: %@\n", __PRETTY_FUNCTION__, document);
@@ -1220,12 +1239,13 @@
 	  RELEASE(document);
 	}
     }
-
+#endif
   return result;
 }
 
 - (id) initForReadingWithData: (NSData*)data
 {
+#if     GNUSTEP_BASE_HAVE_LIBXML
   NSXMLParser *theParser;
   NSData *theData = data;
 
@@ -1255,13 +1275,13 @@
     }
   NS_HANDLER
     {
-      NSLog(@"Exception occured while parsing Xib: %@",[localException reason]);
+      NSLog(@"Exception occurred while parsing Xib: %@",[localException reason]);
       DESTROY(self);
     }
   NS_ENDHANDLER
 
   DESTROY(theParser);
-    
+#endif    
   return self;
 }
 
@@ -1564,10 +1584,11 @@ didStartElement: (NSString*)elementName
 
       if ([type isEqualToString: @"base64-UTF8"])
         {
-          NSData *d = [new dataUsingEncoding: NSASCIIStringEncoding];
-          d = [GSMimeDocument decodeBase64: d];
+          NSData *d = [[NSData alloc] initWithBase64EncodedString: new
+                                                          options: 0];
           new = AUTORELEASE([[NSString alloc] initWithData: d 
                                                   encoding: NSUTF8StringEncoding]);
+          RELEASE(d);
         }
 
       // empty strings are not nil!
@@ -1678,9 +1699,8 @@ didStartElement: (NSString*)elementName
     }
   else if ([@"bytes" isEqualToString: elementName])
     {
-      id new = [[element value] dataUsingEncoding: NSASCIIStringEncoding
-                           allowLossyConversion: NO];
-      new = [GSMimeDocument decodeBase64: new];
+      id new = AUTORELEASE([[NSData alloc] initWithBase64EncodedString: [element value]
+                                                               options: 0]);
 
       if (objID != nil)
         [decoded setObject: new forKey: objID];
