@@ -2915,7 +2915,7 @@ no_soft_invalidation:
 (of selection, wrt range, before change)
       --------------------------
       after     after     location += lengthChange;
-      in        after     length = NSMaxRange(sel)-NSMaxRange(range)-lengthChange; location=NSMaxRange(range);
+      in        after     length = NSMaxRange(sel)-(NSMaxRange(range)-lengthChange); location=NSMaxRange(range);
       in        in        length = 0; location=NSMaxRange(range);
       before    after     length += lengthChange;
       before    in        length = range.location-location;
@@ -2937,7 +2937,7 @@ no_soft_invalidation:
 	{
 	  if (NSMaxRange(_selected_range) > NSMaxRange(range) - lengthChange)
 	    { /* in after */
-	      newRange.length = NSMaxRange(_selected_range) - NSMaxRange(range) - lengthChange;
+	      newRange.length = NSMaxRange(_selected_range) - (NSMaxRange(range) - lengthChange);
 	      newRange.location = NSMaxRange(range);
 	    }
 	  else
@@ -2958,6 +2958,12 @@ no_soft_invalidation:
 	{ /* before before */
 	}
 
+      /* sanity check */
+      if (NSMaxRange(newRange) > [_textStorage length]) 
+        {
+          newRange = NSMakeRange(MIN(range.location, [_textStorage length]), 0);
+        }
+
       /* If there are text views attached to us, let them handle the
       change. */
       if ([self firstTextView])
