@@ -4,7 +4,10 @@
    This class is a subclass of NSTableView which provides the user with a way
    to display tree structured data in an outline format.
    It is particularly useful for show hierarchical data such as a
-   class inheritance tree or any other set of relationships.
+   class inheritance tree or any other set of relationships.<br />
+   NB. While it its illegal to have the same item in the view more than once,
+   it is possible to have multiple equal items since tests for pointer
+   equality are used rather than calls to the -isEqual: method.
    </abstract>
 
    Copyright (C) 2001 Free Software Foundation, Inc.
@@ -62,6 +65,7 @@
 #import "GSGuiPrivate.h"
 #include <math.h>
 
+static NSMapTableKeyCallBacks keyCallBacks;
 static NSNotificationCenter *nc = nil;
 static const int current_version = 1;
 
@@ -141,6 +145,11 @@ static NSImage *unexpandable  = nil;
     {
       [self setVersion: current_version];
       nc = [NSNotificationCenter defaultCenter];
+      /* We need special map table callbacks, to check for identical
+       * objects rather than merely equal objects.
+       */
+      keyCallBacks = NSObjectMapKeyCallBacks;
+      keyCallBacks.isEqual = NSOwnedPointerMapKeyCallBacks.isEqual;
 #if 0
 /* Old Interface Builder style. */
       collapsed    = [NSImage imageNamed: @"common_outlineCollapsed"];
@@ -205,7 +214,7 @@ static NSImage *unexpandable  = nil;
  * Causes the outline column, the column containing the expand/collapse
  * gadget, to resize based on the amount of space needed by widest content.
  */
-- (BOOL)autoResizesOutlineColumn
+- (BOOL) autoResizesOutlineColumn
 {
   return _autoResizesOutlineColumn;
 }
@@ -214,7 +223,7 @@ static NSImage *unexpandable  = nil;
  * Causes the outline column, the column containing the expand/collapse
  * gadget, to resize based on the amount of space needed by widest content.
  */
-- (BOOL)autosaveExpandedItems
+- (BOOL) autosaveExpandedItems
 {
   return _autosaveExpandedItems;
 }
@@ -283,7 +292,7 @@ static NSImage *unexpandable  = nil;
       [self _closeItem: item];
 
       // Send out the notification to let observers know that this has
-      // occured.
+      // occurred.
       [nc postNotificationName: NSOutlineViewItemDidCollapseNotification
           object: self
           userInfo: infoDict];
@@ -338,7 +347,7 @@ static NSImage *unexpandable  = nil;
           [self _openItem: item];
 
           // Send out the notification to let observers know that this has
-          // occured.
+          // occurred.
           [nc postNotificationName: NSOutlineViewItemDidExpandNotification
               object: self
               userInfo: infoDict];
@@ -432,7 +441,11 @@ static NSImage *unexpandable  = nil;
       return YES;
     }
   // Check the array to determine if it is expanded.
-  return([_expandedItems containsObject: item]);
+  if ([_expandedItems indexOfObjectIdenticalTo: item] == NSNotFound)
+    {
+      return NO;
+    }
+  return YES;
 }
 
 /**
@@ -479,7 +492,7 @@ static NSImage *unexpandable  = nil;
 }
 
 /**
- * Returns the parent of the given item or nil
+ * Returns the parent of the given item or nil if the item is not found.
  */
 - (id) parentForItem: (id)item
 {
@@ -492,7 +505,7 @@ static NSImage *unexpandable  = nil;
     {
       NSMutableArray *childArray = NSMapGet(_itemDict, parent);
 
-      if ((index = [childArray indexOfObject: item]) != NSNotFound)
+      if ((index = [childArray indexOfObjectIdenticalTo: item]) != NSNotFound)
         {
           return (parent == [NSNull null]) ? (id)nil : (id)parent;
         }
@@ -532,7 +545,7 @@ static NSImage *unexpandable  = nil;
     {
       NSMutableArray *childArray = NSMapGet(_itemDict, parent);
 
-      if ((index = [childArray indexOfObject: object]) != NSNotFound)
+      if ((index = [childArray indexOfObjectIdenticalTo: object]) != NSNotFound)
         {
           parent = (parent == [NSNull null]) ? (id)nil : (id)parent;
           dsobj = [_dataSource outlineView: self
@@ -573,7 +586,7 @@ static NSImage *unexpandable  = nil;
   if (item == nil)
     return -1;
 
-  row = [_items indexOfObject: item];
+  row = [_items indexOfObjectIdenticalTo: item];
   return (row == NSNotFound) ? -1 : row;
 }
 
@@ -642,7 +655,7 @@ static NSImage *unexpandable  = nil;
 /**
  * Sets the amount, in points, that each level is to be indented by.
  */
-- (void)setIndentationPerLevel: (CGFloat)newIndentLevel
+- (void) setIndentationPerLevel: (CGFloat)newIndentLevel
 {
   _indentationPerLevel = newIndentLevel;
 }
@@ -712,10 +725,10 @@ static NSImage *unexpandable  = nil;
 
   // create a new empty one
   _items = [[NSMutableArray alloc] init];
-  _itemDict = NSCreateMapTable(NSObjectMapKeyCallBacks,
+  _itemDict = NSCreateMapTable(keyCallBacks,
                                NSObjectMapValueCallBacks,
                                64);
-  _levelOfItems = NSCreateMapTable(NSObjectMapKeyCallBacks,
+  _levelOfItems = NSCreateMapTable(keyCallBacks,
                                    NSObjectMapValueCallBacks,
                                    64);
 
@@ -840,8 +853,8 @@ static NSImage *unexpandable  = nil;
       position += _columnOrigins[_clickedColumn];
 
       if ([self isExpandable:item]
-          && location.x >= position
-                && location.x <= position + [image size].width)
+        && location.x >= position
+        && location.x <= position + [image size].width)
         {
           BOOL withChildren =
 	    ([theEvent modifierFlags] & NSAlternateKeyMask) ? YES : NO;
@@ -1069,8 +1082,7 @@ static NSImage *unexpandable  = nil;
 - (void) setDropItem: (id)item
       dropChildIndex: (NSInteger)childIndex
 {
-
-  if (item != nil && [_items indexOfObject: item] == NSNotFound)
+  if (item != nil && [_items indexOfObjectIdenticalTo: item] == NSNotFound)
     {
       /* FIXME raise an exception, or perhaps we should support
        * setting an item which is not visible (inside a collapsed
@@ -1192,7 +1204,7 @@ static NSImage *unexpandable  = nil;
 // TODO: Move a method common to -drapOnRootIndicator and the one below to GSTheme
 - (void) drawDropOnIndicatorWithDropItem: (id)currentDropItem
 {
-  int row = [_items indexOfObject: currentDropItem];
+  int row = [_items indexOfObjectIdenticalTo: currentDropItem];
   int level = [self levelForItem: currentDropItem];
   NSRect newRect = [self frameOfCellAtColumn: 0
                                          row: row];
@@ -1681,7 +1693,8 @@ Also returns the child index relative to this parent. */
           
           // move the drawing rect over like in the drawRow routine...
           drawingRect.origin.x += indentationFactor + 5 + imageRect.size.width;
-          drawingRect.size.width -= indentationFactor + 5 + imageRect.size.width;
+          drawingRect.size.width
+            -= indentationFactor + 5 + imageRect.size.width;
         }
       else
         {
@@ -1809,7 +1822,8 @@ Also returns the child index relative to this parent. */
   if ([_dataSource 
 	respondsToSelector: @selector(outlineView:sortDescriptorsDidChange:)])
     {
-      [_dataSource outlineView: self sortDescriptorsDidChange: oldSortDescriptors];
+      [_dataSource outlineView: self
+      sortDescriptorsDidChange: oldSortDescriptors];
     }
 }
 
@@ -1929,12 +1943,12 @@ Also returns the child index relative to this parent. */
 
 - (void) _initOutlineDefaults
 {
-  _itemDict = NSCreateMapTable(NSObjectMapKeyCallBacks,
+  _itemDict = NSCreateMapTable(keyCallBacks,
                                NSObjectMapValueCallBacks,
                                64);
   _items = [[NSMutableArray alloc] init];
   _expandedItems = [[NSMutableArray alloc] init];
-  _levelOfItems = NSCreateMapTable(NSObjectMapKeyCallBacks,
+  _levelOfItems = NSCreateMapTable(keyCallBacks,
                                    NSObjectMapValueCallBacks,
                                    64);
 
@@ -2078,7 +2092,7 @@ Also returns the child index relative to this parent. */
   // close the item...
   if (item != nil)
     {
-      [_expandedItems removeObject: item];
+      [_expandedItems removeObjectIdenticalTo: item];
     }
 
   // For the close method it doesn't matter what order they are
@@ -2086,7 +2100,7 @@ Also returns the child index relative to this parent. */
   for (i = 0; i < numChildren; i++)
     {
       id child = [removeAll objectAtIndex: i];
-      [_items removeObject: child];
+      [_items removeObjectIdenticalTo: child];
     }
   [self _noteNumberOfRowsChangedBelowItem: item by: -numChildren];
 }
@@ -2107,13 +2121,14 @@ Also returns the child index relative to this parent. */
   // Load the children of the item if needed
   if ([self _isItemLoaded: item] == NO)
     {
-      [self _loadDictionaryStartingWith: item atLevel: [self levelForItem: item]];
+      [self _loadDictionaryStartingWith: item
+                                atLevel: [self levelForItem: item]];
     }
 
   object = NSMapGet(_itemDict, sitem);
   numChildren = numDescendants = [object count];
 
-  insertionPoint = [_items indexOfObject: item];
+  insertionPoint = [_items indexOfObjectIdenticalTo: item];
   if (insertionPoint == NSNotFound)
     {
       insertionPoint = 0;
@@ -2166,8 +2181,8 @@ Also returns the child index relative to this parent. */
 
       [self _removeChildren: child];
       NSMapRemove(_itemDict, child);
-      [_items removeObject: child];
-      [_expandedItems removeObject: child];
+      [_items removeObjectIdenticalTo: child];
+      [_expandedItems removeObjectIdenticalTo: child];
     }
   [anarray removeAllObjects];
   [self _noteNumberOfRowsChangedBelowItem: startitem by: -numChildren];
@@ -2186,7 +2201,7 @@ Also returns the child index relative to this parent. */
   /* Note: We update the selected row indexes directly instead of calling
    * -selectRowIndexes:extendingSelection: to avoid posting bogus selection
    * did change notifications. */
-  rowIndex = [_items indexOfObject: item];
+  rowIndex = [_items indexOfObjectIdenticalTo: item];
   rowIndex = (rowIndex == NSNotFound) ? 0 : rowIndex + 1;
   nextIndex = [_selectedRows indexGreaterThanOrEqualToIndex: rowIndex];
   if (nextIndex != NSNotFound)
