@@ -859,6 +859,8 @@ selectCellWithString: (NSString*)title
 */
 - (void) setTitle: (NSString*)title
 {
+  // keep the window title in sync with the title field
+  [super setTitle:title];
   [_titleField setStringValue: title];
 
   // TODO: Improve the following by managing 
@@ -1108,7 +1110,8 @@ selectCellWithString: (NSString*)title
  */
 - (NSInteger) runModal
 {
-  return [self runModalForDirectory: [self directory] file: [self filename]];
+  return [self runModalForDirectory: [self directory]
+                               file: [[self filename] lastPathComponent]];
 }
 
 - (void) beginSheetModalForWindow:(NSWindow *)window
@@ -1201,7 +1204,7 @@ selectCellWithString: (NSString*)title
 
   if (_allowedFileTypes == nil ||
       [_allowedFileTypes indexOfObject: @""] != NSNotFound)
-    return _fullFileName;
+    return AUTORELEASE([_fullFileName copy]);
 
   /* add file type extension if the file name does not have an extension or
      the file name's extension is not one of the allowed extensions and the
@@ -1216,7 +1219,7 @@ selectCellWithString: (NSString*)title
     }
   else
     {
-      return _fullFileName;
+      return AUTORELEASE([_fullFileName copy]);
     }
 }
 
