@@ -224,6 +224,8 @@ static CGFloat scrollerWidth;
   // For compatibility the ruler should be present but not visible.
   [self setHasHorizontalRuler: YES];
   [self tile];
+  _horizScrollElasticity = NSScrollElasticityAutomatic;
+  _vertScrollElasticity = NSScrollElasticityAutomatic;
 
   [[NSNotificationCenter defaultCenter]
     addObserver: self
@@ -398,6 +400,27 @@ static CGFloat scrollerWidth;
   _autohidesScrollers = flag;
 }
 
+- (NSScrollElasticity)horizontalScrollElasticity
+{
+  return _horizScrollElasticity;
+}
+
+- (void)setHorizontalScrollElasticity:(NSScrollElasticity)value
+{
+  _horizScrollElasticity = value;
+}
+
+- (NSScrollElasticity)verticalScrollElasticity
+{
+  return _vertScrollElasticity;
+}
+
+
+- (void)setVerticalScrollElasticity:(NSScrollElasticity)value
+{
+  _vertScrollElasticity = value;
+}
+
 - (void) scrollWheel: (NSEvent *)theEvent
 {
   NSRect clipViewBounds;
@@ -1511,6 +1534,17 @@ GSOppositeEdge(NSRectEdge edge)
   return _vertScroller;
 }
 
+- (BOOL)allowsMagnification
+{
+  //we need an ivar for this
+  return NO;
+}
+
+- (void)setAllowsMagnification:(BOOL)m
+{
+  //we need an ivar for this
+}
+
 /*
  * NSCoding protocol
  */
